# Generated by Django 3.2.23 on 2024-02-29 14:50

import django.db.models.deletion
from django.db import migrations, models

import wagtail.blocks
import wagtail.contrib.typed_table_block.blocks
import wagtail.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('wagtailcore', '0089_log_entry_data_json_null_to_object'),
    ]

    operations = [
        migrations.CreateModel(
            name='SearchTestPage',
            fields=[
                ('page_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='wagtailcore.page')),
                ('streamfield_with_block', wagtail.fields.StreamField([('block', wagtail.blocks.CharBlock())], use_json_field=True)),
                ('streamfield_with_list', wagtail.fields.StreamField([('list', wagtail.blocks.ListBlock(wagtail.blocks.CharBlock()))], use_json_field=True)),
                ('streamfield_with_struct', wagtail.fields.StreamField([('struct', wagtail.blocks.StructBlock([('givenname', wagtail.blocks.CharBlock()), ('surname', wagtail.blocks.CharBlock())]))], use_json_field=True)),
                ('streamfield_with_table', wagtail.fields.StreamField([('table', wagtail.contrib.typed_table_block.blocks.TypedTableBlock([('text', wagtail.blocks.CharBlock()), ('numeric', wagtail.blocks.FloatBlock())]))], use_json_field=True)),
                ('text', models.TextField(blank=True)),
            ],
            options={
                'abstract': False,
            },
            bases=('wagtailcore.page',),
        ),
    ]
